using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE;
using System.Globalization;

namespace Simplabs.NetDocumentor.Documentation {

	/// <summary>
	/// Factory class that builds a documentor that is suitable for a given language
	/// </summary>
	internal static class DocumentorFactory {

		/// <summary>
		/// A caching dictionary that holds the dockumentors for the supported languages
		/// </summary>
		private static Dictionary<string, IDocumentor> documentorCache = new Dictionary<string,IDocumentor>();

		/// <summary>
		/// Creates a Documentor that is suitable for the given <paramref name="language"/>
		/// </summary>
		/// <param name="language">The language to get a Documentor for</param>
		/// <returns>A Documentor that is suitable for the given <paramref name="language"/></returns>
		public static IDocumentor GetDocumentor(string language) {
			if (documentorCache.ContainsKey(language)) {
				documentorCache[language].Initialize();
				return documentorCache[language];
			}
			IDocumentor documentor;
			switch (language) {
				case CodeModelLanguageConstants.vsCMLanguageMC:
				case CodeModelLanguageConstants.vsCMLanguageVC:
					documentor = new CppDocumentor();
					break;
				case CodeModelLanguageConstants.vsCMLanguageCSharp:
					documentor = new CSharpDocumentor();
					break;
				case CodeModelLanguageConstants.vsCMLanguageVB:
					documentor = new BasicDocumentor();
					break;
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentUICulture, global::Simplabs.NetDocumentor.Properties.Resources.ErrorUnsupportedLanguage, language));
			}
			documentor.Initialize();
			documentorCache.Add(language, documentor);
			return documentor;
		}

	}

}