using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE;
using System.Drawing;
using EnvDTE80;

namespace Simplabs.NetDocumentor.Documentation {

	/// <summary>
	/// Delegate for the <see cref=""/> event
	/// </summary>
	/// <param name="codeElement">The code element that has been documented</param>
	/// <param name="isDocumentationIncomplete">Spesifies whether the documentation comment is still incomplete and requires manual input</param>
	internal delegate void CodeElementDocumentedHandler(CodeElement codeElement, bool isDocumentationIncomplete);

	/// <summary>
	/// Interface that has to be implemented by all language- specific documentors
	/// </summary>
	internal interface IDocumentor {

		/// <summary>
		/// Specifies whether the documentor should suppress the UI
		/// </summary>
		bool SuppressUI {
			get;
			set;
		}

		/// <summary>
		/// Event that is fired whenever a code element is documented
		/// </summary>
		event CodeElementDocumentedHandler ElementDocumented;

		/// <summary>
		/// Initializes the documentor
		/// </summary>
		void Initialize();

		/// <summary>
		/// Generates the XML documentation structure for a given code <paramref name="element"/>
		/// </summary>
		/// <param name="element">The element to document</param>
		/// <param name="documentRecursively">Set to true to document <paramref name="element"/> and alll of its members recursively or set to false to document just <paramref name="element"/></param>
		void DocumentElement(CodeElement2 element, bool documentRecursively);

		/// <summary>
		/// Gets whether <paramref name="element"/> will be documented according to the current settings
		/// </summary>
		/// <param name="element">The element to check for whether it will be documented or not</param>
		bool GetElementWillBeDocumented(CodeElement2 element);

		/// <summary>
		/// Gets whether <paramref name="element"/> can be recursively documented
		/// </summary>
		/// <param name="element">The element to check for whether it can be recursively documented or not</param>
		/// <returns>True if <paramref name="element"/> can be documented recursively, false otherwise</returns>
		bool GetElementCanBeDocumentedRecursively(CodeElement2 element);

		/// <summary>
		/// Gets whether a project item shall be processed by the documentor according to the user's settings
		/// </summary>
		/// <param name="projectItem">The project item to check for whether it will be processed or not</param>
		/// <returns>True if the project item shall be processed according to the user's settings, false otherwise</returns>
		bool GetShallBeProcessed(ProjectItem projectItem);

	}

}