using System;
using System.Collections.Generic;
using System.Text;
using EnvDTE;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Shell;
using System.Globalization;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.VCCodeModel;
using Simplabs.NetDocumentor.Documentation;
using EnvDTE80;

namespace Simplabs.NetDocumentor.IdeIntegration {

	/// <summary>
	/// Task Provider for Documentation Tasks
	/// </summary>
	internal class DocumentationTaskProvider : TaskProvider {

		/// <summary>
		/// The category of the task provider
		/// </summary>
		private VSTASKCATEGORY category;

		/// <summary>
		/// Initializes a new DocumentationTaskProvider
		/// </summary>
		internal DocumentationTaskProvider()
			: base() {
			category = VSTASKCATEGORY.CAT_COMMENTS;
		}

		/// <summary>
		/// Adds a new documentation task to the task list
		/// </summary>
		/// <param name="codeElement">The code element to add the atsk for</param>
		internal void AddNewTask(CodeElement codeElement) {
			AddTask(new DocumentationTask(codeElement));
		}

		/// <summary>
		/// Shows all tasks the task provider provides
		/// </summary>
		internal void ShowAllTasks() {
			FlushAndRefresh();
			taskList.AutoFilter(category);
		}

		/// <summary>
		/// A custom documentation task
		/// </summary>
		internal class DocumentationTask : Task {

			/// <summary>
			/// The code element the task is associated to
			/// </summary>
			private CodeElement theCodeElement;

			/// <summary>
			/// Initializes a new DocumentationTask
			/// </summary>
			/// <param name="codeElement">The code element the task is associated to</param>
			internal DocumentationTask(CodeElement codeElement)
				: base(codeElement.ProjectItem, string.Empty, codeElement.StartPoint.Line, TaskPriority.Normal, VSTASKCATEGORY.CAT_COMMENTS)
			{
				theCodeElement = codeElement;
				base.put_Text(string.Format(
					CultureInfo.CurrentUICulture,
					global::Simplabs.NetDocumentor.Properties.Resources.TaskItemCaption,
					codeElement.FullName
				));
			}

			/// <summary>
			/// Navigates to the code element the task is associated to
			/// </summary>
			/// <returns>VSConstants.S_OK on success, VSConstants.S_FAIL otherwise</returns>
			public override int NavigateTo() {
				try {
					Window projectItemWindow = null;
					if (!taskProjectItem.get_IsOpen(EnvDTE.Constants.vsViewKindAny)) {
						projectItemWindow = taskProjectItem.Open(EnvDTE.Constants.vsViewKindCode);
					}
					foreach (Window window in taskProjectItem.Document.Windows) {
						projectItemWindow = window;
						break;
					}
					projectItemWindow.Activate();
					projectItemWindow.Visible = true;
					TextSelection selection = projectItemWindow.Document.Selection as TextSelection;
					EditPoint selectionPoint = ((TextPoint)selection.ActivePoint).CreateEditPoint();
					selectionPoint.MoveToLineAndOffset(theCodeElement.StartPoint.Line, 1);
					selectionPoint.TryToShow(vsPaneShowHow.vsPaneShowCentered, null);
				} catch (Exception) {
					return VSConstants.S_FALSE;
				}
				return VSConstants.S_OK;
			}

			/// <summary>
			/// Gets the line the task refers to
			/// </summary>
			/// <param name="piLine">Is set to the line the task refers to</param>
			/// <returns>VSConstants.S_OK on success, VSConstants.S_FAIL otherwise</returns>
			public override int Line(out int piLine) {
				piLine = theCodeElement.StartPoint.Line - 1;
				return VSConstants.S_OK;
			}

		}

	}

}