using System;
using System.Collections.Generic;
using System.Text;
using IServiceProvider = Microsoft.VisualStudio.OLE.Interop.IServiceProvider;
using Microsoft.VisualStudio;
using System.Runtime.InteropServices;
using System.Globalization;

namespace Simplabs.NetDocumentor.IdeIntegration {

	/// <summary>
	/// Utility class that provides access to the services of the IDE
	/// </summary>
	internal static class ServiceProvider {

		/// <summary>
		/// Gets a service of type <typeparamref name="T"/>
		/// </summary>
		/// <typeparam name="T">The type of the service to get</typeparam>
		/// <returns>The service of type <typeparamref name="T"/></returns>
		internal static T GetService<T>() where T : class {
			try {
				IServiceProvider provider;
				provider = Addin.ApplicationObject as IServiceProvider;
				IntPtr outputObject;
				Guid serviceGuid = typeof(T).GUID;
				int result = provider.QueryService(ref serviceGuid, ref serviceGuid, out outputObject);
				if (result != VSConstants.S_OK) {
					Marshal.ThrowExceptionForHR(result);
					return null;
				}
				object service = Marshal.GetObjectForIUnknown(outputObject);
				Marshal.Release(outputObject);
				return service as T;
			} catch (Exception exception) {
				throw new IntegrationException(
					string.Format(
						CultureInfo.CurrentUICulture,
						global::Simplabs.NetDocumentor.Properties.Resources.ErrorCannotGetService,
						typeof(T).FullName
					),
					exception
				);
			}
		}

	}

}