using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace Simplabs.NetDocumentor.Settings {

	/// <summary>
	/// A feature
	/// </summary>
	internal class Feature {

		/// <summary>
		/// The identifying key of the feature
		/// </summary>
		private string key;

		/// <summary>
		/// The localized name of the feature
		/// </summary>
		private string name;

		/// <summary>
		/// The localized description of the feature
		/// </summary>
		private string description;

		/// <summary>
		/// The property that holds whether the feature is enabled or not
		/// </summary>
		private PropertyInfo isEnabledProperty;

		/// <summary>
		/// Initializes a new Feature
		/// </summary>
		/// <param name="featureKey">The identifying key of the feature</param>
		/// <param name="featureName">The localized name of the feature</param>
		/// <param name="featureDescription">The localized description of the feature</param>
		/// <param name="enabledProperty">The property that provides the information whether the feature is enabled or not according to the user's settings</param>
		internal Feature(string featureKey, string featureName, string featureDescription, PropertyInfo enabledProperty) {
			key = featureKey;
			name = featureName;
			description = featureDescription;
			isEnabledProperty = enabledProperty;
		}

		/// <summary>
		/// Gets the identifying key of the feature
		/// </summary>
		public string Key {
			get {
				return key;
			}
		}

		/// <summary>
		/// Gets the localized name of the feature
		/// </summary>
		public string Name {
			get {
				return name;
			}
		}

		/// <summary>
		/// Gets the localized description of the feature
		/// </summary>
		public string Description {
			get {
				return description;
			}
		}

		/// <summary>
		/// Gets or sets whether the feature is enabled or not
		/// </summary>
		internal bool IsEnabled {
			get {
				return (bool)isEnabledProperty.GetValue(global::Simplabs.NetDocumentor.Properties.FeatureSettings.Default, null);
			}
			set {
				isEnabledProperty.SetValue(global::Simplabs.NetDocumentor.Properties.FeatureSettings.Default, value, null);
			}
		}

	}

}