using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using Simplabs.NetDocumentor.Documentation;

namespace Simplabs.NetDocumentor.Settings {

	/// <summary>
	/// Holds a list of the features the AddIn proovides
	/// </summary>
	internal static class FeaturesStore {

		/// <summary>
		/// The suffix for resources that hold names of features
		/// </summary>
		private static string featureNameResourceSuffix = "Name";

		/// <summary>
		/// The suffix for resources that hold descriptions of features
		/// </summary>
		private static string featureDescriptionResourceSuffix = "Description";

		/// <summary>
		/// The list of feature descriptors that describe the features the AddIn provides
		/// </summary>
		private static List<Feature> features = new List<Feature>();

		/// <summary>
		/// Initializes the list of features the AddIn provides
		/// </summary>
		static FeaturesStore() {
			features.Clear();
			foreach (PropertyInfo property in typeof(global::Simplabs.NetDocumentor.Properties.FeatureSettings).GetProperties(BindingFlags.DeclaredOnly | BindingFlags.Instance | BindingFlags.Public)) {
				features.Add(
					new Feature(
						property.Name,
						global::Simplabs.NetDocumentor.Properties.FeatureResources.ResourceManager.GetString(property.Name + featureNameResourceSuffix),
						global::Simplabs.NetDocumentor.Properties.FeatureResources.ResourceManager.GetString(property.Name + featureDescriptionResourceSuffix),
						property
					)
				);
			}
		}

		/// <summary>
		/// Gets the list of features in the feature set with the identifying key <paramref name="featureSetKey"/>
		/// </summary>
		/// <param name="featureSet">The feature set to get the features for</param>
		internal static List<Feature> GetFeatures(string languagePrefix) {
			List<Feature> languageFeatures = new List<Feature>();
			foreach (Feature feature in features) {
				if (feature.Key.StartsWith(languagePrefix)) {
					languageFeatures.Add(feature);
				}
			}
			return languageFeatures;
		}

	}

}