using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace Simplabs.NetDocumentor.Settings {

	/// <summary>
	/// A text snippet
	/// </summary>
	internal class TextSnippet {

		/// <summary>
		/// The name of the text snippet
		/// </summary>
		private string name;

		/// <summary>
		/// The description of the text snippet
		/// </summary>
		private string description;

		/// <summary>
		/// The property that holds the actual value of the text snippet
		/// </summary>
		private PropertyInfo settingsProperty;

		/// <summary>
		/// Initializes a new TextSnipppet
		/// </summary>
		/// <param name="snippetName">The name of the text snippet</param>
		/// <param name="snippetDescription">The description of the text snippet</param>
		/// <param name="valueProperty">The property that provides the actual value of the text snippet</param>
		internal TextSnippet(string snippetName, string snippetDescription, PropertyInfo valueProperty) {
			name = snippetName;
			description = snippetDescription;
			settingsProperty = valueProperty;
		}

		/// <summary>
		/// Gets the name of the text snippet
		/// </summary>
		public string Name {
			get {
				return name;
			}
		}

		/// <summary>
		/// Gets the description of the text snippet
		/// </summary>
		public string Description {
			get {
				return description;
			}
		}

		/// <summary>
		/// Gets or sets the actual value of the text snippet
		/// </summary>
		internal string Text {
			get {
				return (string)settingsProperty.GetValue(global::Simplabs.NetDocumentor.Properties.TextSnippetSettings.Default, null);
			}
			set {
				settingsProperty.SetValue(global::Simplabs.NetDocumentor.Properties.TextSnippetSettings.Default, value, null);
			}
		}

	}

}