using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using Simplabs.NetDocumentor.Documentation;

namespace Simplabs.NetDocumentor.Settings {

	/// <summary>
	/// Holds a list of the features the AddIn proovides
	/// </summary>
	internal static class TextSnippetsStore {

		/// <summary>
		/// The suffix for resources that hold names of text snippets
		/// </summary>
		private static string snippetNameResourceSuffix = "Name";

		/// <summary>
		/// The suffix for resources that hold descriptions of text snippets
		/// </summary>
		private static string snippetDescriptionResourceSuffix = "Description";

		/// <summary>
		/// The list of text snippets descriptors that describe the text snippets the AddIn provides
		/// </summary>
		private static List<TextSnippet> snippets = new List<TextSnippet>();

		/// <summary>
		/// Initializes the list of text snippets the AddIn provides
		/// </summary>
		static TextSnippetsStore() {
			snippets.Clear();
			foreach (PropertyInfo property in typeof(global::Simplabs.NetDocumentor.Properties.TextSnippetSettings).GetProperties(BindingFlags.DeclaredOnly | BindingFlags.Instance | BindingFlags.Public)) {
				snippets.Add(
					new TextSnippet(
						global::Simplabs.NetDocumentor.Properties.TextSnippetResources.ResourceManager.GetString(property.Name + snippetNameResourceSuffix),
						global::Simplabs.NetDocumentor.Properties.TextSnippetResources.ResourceManager.GetString(property.Name + snippetDescriptionResourceSuffix),
						property
					)
				);
			}
		}

		/// <summary>
		/// Gets the list of text snippets
		/// </summary>
		internal static List<TextSnippet> GetSnippets() {
			return snippets;
		}

	}

}