using System;
using System.Collections.Generic;
using System.Text;
using Simplabs.NetDocumentor.Settings;
using System.Windows.Forms;
using Simplabs.NetDocumentor.Documentation;
using System.Globalization;

namespace Simplabs.NetDocumentor.UI {

	/// <summary>
	/// The options page for C, C++ and C++/CLI settings
	/// </summary>
	public sealed class CppOptionsPage : OptionsPage {

		/// <summary>
		/// The key of the feature set the optiions page configures
		/// </summary>
		private const string featureSetKey = "CppDocumentation";

		/// <summary>
		/// Initializes a new CppOptionsPage
		/// </summary>
		public CppOptionsPage() : base() {
			InitializeComponent();
		}

		/// <summary>
		/// Performs manual initialization of the form
		/// </summary>
		protected override void ManualInitialize() {
			Features = FeaturesStore.GetFeatures(CppDocumentor.LanguagePrefix);
			labelCaption.Text = string.Format(
				CultureInfo.CurrentUICulture,
				labelCaption.Text,
				global::Simplabs.NetDocumentor.Properties.Resources.CppDocumentationName
			);
			radioButtonDeclaration.Text = global::Simplabs.NetDocumentor.Properties.Resources.CppOptionsPageDocumentAtDeclarationCaption;
			radioButtonDefinition.Text = global::Simplabs.NetDocumentor.Properties.Resources.CppOptionsPageDocumentAtDefinitionCaption;
			groupBoxCustomOptions.Text = global::Simplabs.NetDocumentor.Properties.Resources.CppOptionsPageDocumentationPositionCaption;
			if (global::Simplabs.NetDocumentor.Properties.LanguageSpecificSettings.Default.CppDocumentAtDeclaration) {
				radioButtonDeclaration.Checked = true;
			} else if (global::Simplabs.NetDocumentor.Properties.LanguageSpecificSettings.Default.CppDocumentAtDefinition) {
				radioButtonDefinition.Checked = true;
			}
		}

		/// <summary>
		/// Saves custom options
		/// </summary>
		protected override void SaveCustomOptions() {
			global::Simplabs.NetDocumentor.Properties.LanguageSpecificSettings.Default.CppDocumentAtDeclaration = false;
			global::Simplabs.NetDocumentor.Properties.LanguageSpecificSettings.Default.CppDocumentAtDefinition = false;
			if (radioButtonDeclaration.Checked) {
				global::Simplabs.NetDocumentor.Properties.LanguageSpecificSettings.Default.CppDocumentAtDeclaration = true;
			} else if (radioButtonDefinition.Checked) {
				global::Simplabs.NetDocumentor.Properties.LanguageSpecificSettings.Default.CppDocumentAtDefinition = true;
			}
		}

		#region Component Designer generated code

		private System.Windows.Forms.RadioButton radioButtonDeclaration;
		private System.Windows.Forms.RadioButton radioButtonDefinition;
		private System.Windows.Forms.FlowLayoutPanel flowLayoutPanelPositionOfDocumentation;
		private GroupBox groupBoxCustomOptions;

		private void InitializeComponent() {
			this.radioButtonDeclaration = new System.Windows.Forms.RadioButton();
			this.radioButtonDefinition = new System.Windows.Forms.RadioButton();
			this.flowLayoutPanelPositionOfDocumentation = new System.Windows.Forms.FlowLayoutPanel();
			this.groupBoxCustomOptions = new System.Windows.Forms.GroupBox();
			this.flowLayoutPanelPositionOfDocumentation.SuspendLayout();
			this.groupBoxCustomOptions.SuspendLayout();
			this.SuspendLayout();
			// 
			// treeViewOptions
			// 
			this.treeViewOptions.LineColor = System.Drawing.Color.Black;
			this.treeViewOptions.Size = new System.Drawing.Size(389, 126);
			// 
			// groupBoxFeatureDescription
			// 
			this.groupBoxFeatureDescription.Location = new System.Drawing.Point(0, 152);
			// 
			// radioButtonDeclaration
			// 
			this.radioButtonDeclaration.AutoSize = true;
			this.radioButtonDeclaration.Location = new System.Drawing.Point(3, 3);
			this.radioButtonDeclaration.Name = "radioButtonDeclaration";
			this.radioButtonDeclaration.Size = new System.Drawing.Size(14, 13);
			this.radioButtonDeclaration.TabIndex = 1;
			this.radioButtonDeclaration.TabStop = true;
			this.radioButtonDeclaration.UseVisualStyleBackColor = true;
			// 
			// radioButtonDefinition
			// 
			this.radioButtonDefinition.AutoSize = true;
			this.radioButtonDefinition.Location = new System.Drawing.Point(23, 3);
			this.radioButtonDefinition.Name = "radioButtonDefinition";
			this.radioButtonDefinition.Size = new System.Drawing.Size(14, 13);
			this.radioButtonDefinition.TabIndex = 2;
			this.radioButtonDefinition.TabStop = true;
			this.radioButtonDefinition.UseVisualStyleBackColor = true;
			// 
			// flowLayoutPanelPositionOfDeclaration
			// 
			this.flowLayoutPanelPositionOfDocumentation.Controls.Add(this.radioButtonDeclaration);
			this.flowLayoutPanelPositionOfDocumentation.Controls.Add(this.radioButtonDefinition);
			this.flowLayoutPanelPositionOfDocumentation.Location = new System.Drawing.Point(6, 19);
			this.flowLayoutPanelPositionOfDocumentation.Name = "flowLayoutPanelPositionOfDeclaration";
			this.flowLayoutPanelPositionOfDocumentation.Size = new System.Drawing.Size(378, 22);
			this.flowLayoutPanelPositionOfDocumentation.TabIndex = 4;
			this.flowLayoutPanelPositionOfDocumentation.WrapContents = false;
			// 
			// groupBoxCustomOptions
			// 
			this.groupBoxCustomOptions.Controls.Add(this.flowLayoutPanelPositionOfDocumentation);
			this.groupBoxCustomOptions.Location = new System.Drawing.Point(0, 236);
			this.groupBoxCustomOptions.Name = "groupBoxCustomOptions";
			this.groupBoxCustomOptions.Size = new System.Drawing.Size(393, 50);
			this.groupBoxCustomOptions.TabIndex = 4;
			this.groupBoxCustomOptions.TabStop = false;
			this.groupBoxCustomOptions.Text = "groupBox1";
			// 
			// CppOptionsPage
			// 
			this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
			this.Controls.Add(this.groupBoxCustomOptions);
			this.Name = "CppOptionsPage";
			this.Controls.SetChildIndex(this.groupBoxFeatureDescription, 0);
			this.Controls.SetChildIndex(this.groupBoxCustomOptions, 0);
			this.Controls.SetChildIndex(this.treeViewOptions, 0);
			this.Controls.SetChildIndex(this.labelCaption, 0);
			this.flowLayoutPanelPositionOfDocumentation.ResumeLayout(false);
			this.flowLayoutPanelPositionOfDocumentation.PerformLayout();
			this.groupBoxCustomOptions.ResumeLayout(false);
			this.ResumeLayout(false);
			this.PerformLayout();
		}

		#endregion

	}

}