using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Simplabs.NetDocumentor.Settings;

namespace Simplabs.NetDocumentor.UI {

	/// <summary>
	/// The options page for text snippets
	/// </summary>
	public partial class TextSnippetsOptionsPage : UserControl, EnvDTE.IDTToolsOptionsPage {

		/// <summary>
		/// Initializes a new TextSnippetsOptionsPage
		/// </summary>
		public TextSnippetsOptionsPage() {
			InitializeComponent();
			labelUnavailable.Text = global::Simplabs.NetDocumentor.Properties.Resources.OptionsPageAddinUnavailable;
			groupBoxSnippetDescription.Text = global::Simplabs.NetDocumentor.Properties.Resources.OptionsPageSnippetDescriptionCaption;
			dataGridViewTextSnippets.Columns[0].HeaderText = global::Simplabs.NetDocumentor.Properties.Resources.OptionsPageSnippetNameColumnCaption;
			dataGridViewTextSnippets.Columns[1].HeaderText = global::Simplabs.NetDocumentor.Properties.Resources.OptionsPageSnippetValueColumnCaption;
		}

		/// <summary>
		/// Initializes the tree view (sets up the nodes with appropriate caption and state)
		/// </summary>
		private void InitializeDataGrid() {
			List<TextSnippet> snippets = TextSnippetsStore.GetSnippets();
			dataGridViewTextSnippets.Rows.Clear();
			foreach (TextSnippet snippet in snippets) {
				dataGridViewTextSnippets.Rows.Add(snippet.Name, snippet.Text);
				dataGridViewTextSnippets.RowEnter += new DataGridViewCellEventHandler(delegate(object sender, DataGridViewCellEventArgs e) {
					labelSnippetDescription.Text = snippets[e.RowIndex].Description;
				});
				dataGridViewTextSnippets.CellEndEdit += new DataGridViewCellEventHandler(delegate(object sender, DataGridViewCellEventArgs e) {
					snippets[e.RowIndex].Text = dataGridViewTextSnippets.Rows[e.RowIndex].Cells[e.ColumnIndex].Value.ToString();
				});
			}
		}

		/// <summary>
		/// Returns an object containing all properties contained on the specified custom Tools Options page.
		/// </summary>
		/// <param name="PropertiesObject">The Tools Options object.</param>
		void EnvDTE.IDTToolsOptionsPage.GetProperties(ref object PropertiesObject) {
		}

		/// <summary>
		/// Handles the AfterCreated event of the options page
		/// </summary>
		/// <param name="DTEObject">The parent object of the Tools Options page, namely, a EnvDTE.DTE object.</param>
		void EnvDTE.IDTToolsOptionsPage.OnAfterCreated(EnvDTE.DTE DTEObject) {
			if (!Addin.IsConnected) {
				panelUnavailable.Visible = true;
			} else {
				panelUnavailable.Visible = false;
				InitializeDataGrid();
			}
		}

		/// <summary>
		/// Handles the Cancel event of the options page
		/// </summary>
		void EnvDTE.IDTToolsOptionsPage.OnCancel() {
			global::Simplabs.NetDocumentor.Properties.TextSnippetSettings.Default.Reload();
		}

		/// <summary>
		/// Handles the Help event of the options page
		/// </summary>
		void EnvDTE.IDTToolsOptionsPage.OnHelp() {
		}

		/// <summary>
		/// Handles the OK event of the options page
		/// </summary>
		void EnvDTE.IDTToolsOptionsPage.OnOK() {
			global::Simplabs.NetDocumentor.Properties.TextSnippetSettings.Default.Save();
		}

	}

}