//-----------------------------------------------------------------------
//  This file is part of the Microsoft .NET SDK Code Samples.
// 
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
//This source code is intended only as a supplement to Microsoft
//Development Tools and/or on-line documentation.  See these other
//materials for detailed information regarding Microsoft code samples.
// 
//THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
//KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
//PARTICULAR PURPOSE.
//-----------------------------------------------------------------------

using System;
using System.Globalization;
using System.Text;

namespace Microsoft.VisualStudio.Samples.TextEncoding.EncodingApplication
{
    /// <summary>
    /// Class to encapsulate encoding options of Encoding type, Fallback mechanism and Normalization scheme
    /// </summary>
    /// <seealso cref="T:Microsoft.VisualStudio.Samples.TextEncoding.EncodingApplication.ITest"/>
	public class EncoderInfo : ITest
	{
        /// <summary>
        /// </summary>
		public static string NormalizationNone = "None";
        /// <summary>
        /// </summary>
		public static string[] FallbackSchemes = { "Replacement", "Exception", "Best Fit" };

        /// <summary>
        /// </summary>
		private int encodingCodePage = 0;

        /// <summary>
        /// </summary>
		private Encoding encoding = Encoding.Default;

        /// <summary>
        /// </summary>
		private EncoderFallback encFallback = EncoderFallback.ReplacementFallback;

        /// <summary>
        /// </summary>
		private string normalizationForm;

        /// <summary>
        /// Initialisiert eine neue Microsoft.VisualStudio.Samples.TextEncoding.EncodingApplication.EncoderInfo
        /// </summary>
		public EncoderInfo()
		{
			//Initialize encoding to UTF8
			SetEncodingCodePage(Encoding.UTF8.CodePage);

			//Initialize fallback scheme to replacement
			SetEncoderFallback(FallbackSchemes[0]);

			//Initialize normalization scheme to none
			SetNormalizationForm(NormalizationNone);

		}

        /// <summary>
        /// Liest die
        /// </summary>
        /// <value></value>
		public Encoding Encoding
		{
			get
			{
				return (Encoding)this.encoding.Clone();
			}
		}

        /// <summary>
        /// </summary>
        /// <param name="encodingCodePage"></param>
		public void SetEncodingCodePage(int encodingCodePage)
		{
			this.encodingCodePage = encodingCodePage;
			this.encoding = Encoding.GetEncoding(encodingCodePage, encFallback, DecoderFallback.ReplacementFallback);
			EncoderBestFitFallback bestFit = encFallback as EncoderBestFitFallback;
			if (bestFit != null)            
			{
				bestFit.ParentEncoding = encoding;
			}            
		}

        /// <summary>
        /// </summary>
        /// <param name="strFallback"></param>
		public void SetEncoderFallback(string strFallback)
		{            
			// Replacement
			if (strFallback == FallbackSchemes[0])
			{
				this.encFallback = new EncoderReplacementFallback("?");
			}
			// Exception            
			else if (strFallback == FallbackSchemes[1])
			{
				this.encFallback = new EncoderExceptionFallback();
			}
			// Best Fit
			else
			{
				this.encFallback = new EncoderBestFitFallback(this.encoding);
			}
			this.encoding.EncoderFallback = this.encFallback;            
		}

        /// <summary>
        /// </summary>
        /// <param name="strNormalization"></param>
		public void SetNormalizationForm(string strNormalization)
		{
			normalizationForm = strNormalization;
		}

        /// <summary>
        /// </summary>
        /// <param name="inputString"></param>
        /// <returns></returns>
		public string GetNormalizedString(string inputString)
		{
			if (normalizationForm != NormalizationNone)
			{
				NormalizationForm n = (NormalizationForm) Enum.Parse(typeof(NormalizationForm), normalizationForm);
				//Normalize string
				inputString = inputString.Normalize(n);
			}

			return inputString;
		}

        #region ITest Members

        /// <summary>
        /// </summary>
        /// <seealso cref="Test(int)"/>
        /// <exception cref="Exception"></exception>
        public void Test()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region ITest Members

        /// <summary>
        /// </summary>
        void ITest.Test()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// </summary>
        /// <param name="test"></param>
        void ITest.Test(EncodingApplication.Test test)
        {
            throw new Exception("The method or operation is not implemented.");
        }
       

        #endregion
    }
}
