using System;
using System.Collections;

//
// WHAT'S THIS?
//
//	This is some source code for showing the features of GhostDoc
//	from version 1.0 to 1.9.5. The new features of version 2.0 are
//	demonstrated in the source file NewIn2.0.0.cs
//
// WHAT YOU CAN DO
//
//	Move the cursor around and try the "Document this" command
//	(available either by context menu or hotkey if you have assigned
//	one) on language elements like properties and methods.
//
// NOTE:
//
//	Whenever you see square brackets in the generated text, GhostDoc
//	tries to tell you that it is not quite sure what to do with the
//	words between the brackets. It's then up to the develoepr to move
//	the words around and/or edit the text. On a sidenote, seeing
//	square brackets in someone's documentation comments is generally
//	NOT a good sign ;-)
//
namespace Demo
{

    /// <summary>
    /// </summary>
    /// <seealso cref="T:System.Collections.IEnumerable"/>
    /// <seealso cref="T:Demo.ITest"/>
    public class Demo : IEnumerable, ITest
	{
		//=============================================
		// Methods
		//=============================================

        /// <summary>
        /// </summary>
        /// <returns></returns>
        public string GetNextItem() // default rule
		{
			return null;
		}

        /// <summary>
        /// </summary>
		public void PerformSomeAction() // default rule, no "the"
		{
		}

        /// <summary>
        /// </summary>
		public void DoSomething() // default rule, no "the", Do -> Does
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="targetDirectory"></param>
		public void ChangeDirectory(string targetDirectory) // default rule, with parameters
		{
		}

        /// <summary>
        /// </summary>
        /// <returns></returns>
		public string BuildHtmlText() // default rule, automatic acronym HTML (all consonants)
		{
			return "";
		}

        /// <summary>
        /// </summary>
		public void InitializeGui() // default rule, configured acronym GUI
		{
		}

        /// <summary>
        /// </summary>
        /// <returns></returns>
		public int DetermineInitialPageBufferSize() // "of the" reordering of the method name
		{
			return 0;
		}

        /// <summary>
        /// </summary>
        /// <param name="initialPageBufferSize"></param>
        /// <returns></returns>
		public int DeterminePageBufferSize(int initialPageBufferSize) // additional "of the" reordering of the parameter name
		{
			return 0;
		}

        /// <summary>
        /// </summary>
		public void Initialize()
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="item"></param>
		public void Add(object item)
		{
		}

		//
		// New in version 1.2.0
		//

        /// <summary>
        /// </summary>
        /// <returns></returns>
		public bool IsModified()
		{
			return false;
		}

        /// <summary>
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
		public bool HasChanged(object item) // one word after "Has"
		{
			return false;
		}

        /// <summary>
        /// </summary>
        /// <returns></returns>
		public bool HasContextChanged() // multiple words adter "Has"
		{
			// This is an example where GhostDoc puts words in square
			// brackets as it does not actually understand the meaning
			// of the words.
			return false;
		}

        /// <summary>
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
		public bool IsConnectionPossible(object source)
		{
			// And here's another example of the usage of square brackets.
			// The idea is to give the developer a slight head start by
			// writing words to the summary that then may be moved or
			// edited by the developer. If you prefer a different behavior,
			// you can customize the text generation in the configuration
			// dialog (Tools -> GhostDoc -> Configure GhostDoc). In this
			// example you would edit the rule "State check (Is..., Hst...)".
			return false;
		}

        /// <summary>
        /// </summary>
        /// <returns></returns>
		public bool CanRestart()
		{
			return false;
		}

        /// <summary>
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
		public bool CanReceive(object data)
		{
			return false;
		}

        /// <summary>
        /// </summary>
        /// <returns></returns>
		public bool CanExtendDeadline()
		{
			return false;
		}

        /// <summary>
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
		public bool CanConnectTo(object source)
		{
			return false;
		}

		//=============================================
		// Constructors
		//=============================================

        /// <summary>
        /// Initializes a new Demo.Demo.
        /// </summary>
		public Demo() // A default constructor
		{
		}

        /// <summary>
        /// Initializes a new Demo.Demo.
        /// </summary>
        /// <param name="parameter"></param>
		public Demo( string parameter ) // Constructor with parameter
		{
		}

		//=============================================
		// Properties
		//=============================================

        /// <summary>
        /// Gets the
        /// </summary>
        /// <value></value>
		public string HomeDirectory // default rule
		{
			get { return null; }
		}

        /// <summary>
        /// Gets the
        /// </summary>
        /// <value></value>
		public string FullFileName // default rule, "of the" reordering
		{
			get { return null; }
		}

        /// <summary>
        /// Gets the
        /// </summary>
        /// <value></value>
		public bool JustTesting // boolean property
		{
			get { return true; }
		}

        /// <summary>
        /// Gets or sets the
        /// </summary>
        /// <value></value>
		public bool Cool // boolean property, single word
		{
			get { return true; }
			set { ; }
		}

		//
		// New in version 1.2.0
		//

        /// <summary>
        /// Gets the
        /// </summary>
        /// <value></value>
		public bool IsReady
		{
			get { return false; }
		}

        /// <summary>
        /// Gets the
        /// </summary>
        /// <value></value>
		public bool HasPendingChanges
		{
			get { return false; }
		}

		//=============================================
		// Indexers
		//=============================================

        /// <summary>
        /// Gets the
        /// </summary>
        /// <param name="name"></param>
        /// <value></value>
		public string this[string name] // default rule
		{
			get { return null; }
		}

        /// <summary>
        /// Gets the
        /// </summary>
        /// <param name="index"></param>
        /// <value></value>
		public string this[int index] // default rule, special handling of "index"
		{
			get { return null; }
		}

		//===================================================
		// Various other features that came with 1.3.0 and later versions
		//===================================================

        /// <summary>
        /// </summary>
		~Demo() // Documentation for Finalize
		{
		}

        /// <summary>
        /// Initializes the Demo.Demo Type
        /// </summary>
		static Demo() // Documentation for static constructors
		{
		}

        /// <summary>
        /// </summary>
		protected virtual void OnAction() // Documentation for "On..." methods (1)
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="e"></param>
		protected virtual void OnClick(EventArgs e) // Documentation for "On..." methods (2)
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="first"></param>
        /// <param name="second"></param>
        /// <param name="ignoreCase"></param>
		public void CompareString(string first, string second, bool ignoreCase) // boolean parameter
		{
		}

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"></see> is equal to the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"></see> to compare with the current <see cref="T:System.Object"></see>.</param>
        /// <returns>true if the specified <see cref="T:System.Object"></see> is equal to the current <see cref="T:System.Object"></see>; otherwise, false.</returns>
        /// <filterpriority>2</filterpriority>
		public override bool Equals(object obj) // Documentation inherited from base class
		{
			return base.Equals(obj);
		}

        /// <summary>
        /// Serves as a hash function for a particular type. <see cref="M:System.Object.GetHashCode"></see> is suitable for use in hashing algorithms and data structures like a hash table.
        /// </summary>
        /// <returns>A hash code for the current <see cref="T:System.Object"></see>.</returns>
        /// <filterpriority>2</filterpriority>
		public override int GetHashCode() // Documentation inherited from base class
		{
			return base.GetHashCode();
		}

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>An <see cref="T:System.Collections.IEnumerator"></see> object that can be used to iterate through the collection.</returns>
        /// <filterpriority>2</filterpriority>
		public IEnumerator GetEnumerator() // Documentation provided by implemented interface
		{
			return null;
		}

        /// <summary>
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
		public void MyButton_Click(object sender, EventArgs e) // event handler method
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="FirstParameter"></param>
        /// <param name="SecondParameter"></param>
        /// <param name="ThirdParameter"></param>
		public void DemonstrateUpdateFeature(string FirstParameter, string SecondParameter, string ThirdParameter)
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        object Test(int a)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #region ITest Members

        /// <summary>
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        object ITest.Test(int a)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// </summary>
        /// <param name="b"></param>
        /// <returns></returns>
        object ITest.Test(float b)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// fdsfdfsf
        /// </summary>
        /// <param name="b">fdf</param>
        /// <param name="c">fd</param>
        /// <returns>fdsf</returns>
        object ITest.Test(float b, string c)
        {
            return null;
        }

        /// <summary>
        /// Occurs when
        /// </summary>
        event EventHandler ITest.TestOccured
        {
            add { throw new Exception("The method or operation is not implemented."); }
            remove { throw new Exception("The method or operation is not implemented."); }
        }

        /// <summary>
        /// Occurs when
        /// </summary>
        event EventHandler<EventArgs> ITest.TestOccured2
        {
            add { throw new Exception("The method or operation is not implemented."); }
            remove { throw new Exception("The method or operation is not implemented."); }
        }

        /// <summary>
        /// Gets or sets the
        /// </summary>
        /// <param name="a"></param>
        /// <value></value>
        Demo ITest.this[int a]
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        /// <summary>
        /// Gets or sets the
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <value></value>
        Demo ITest.this[int a, int b]
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        #endregion

        #region IEquatable<string> Members

        /// <summary>
        /// Indicates whether the current object is equal to another object of the same type.
        /// </summary>
        /// <param name="other">An object to compare with this object.</param>
        /// <returns>true if the current object is equal to the other parameter; otherwise, false.</returns>
        bool IEquatable<string>.Equals(string other)
        {
            return false;
        }

        #endregion
    }
}
