using System;
using System.Collections;
using System.Collections.Generic;

//
// WHAT'S THIS?
//
//	This is some source code for showing the new features of
//	GhostDoc version 2.0. The features of version 1.0 to 1.9.5
//	are demonstrated in the source file Demo.cs
//
// WHAT YOU CAN DO
//
//	Move the cursor around and try the "Document this" command
//	(available either via context menu or hotkey if you have assigned
//	one) on various elements of the source code.
//
// NOTE:
//
//	Classes, interfaces and structs are handled by text generation
//	rules, so you can add your own custom rules, either for handling
//	specific cases, or for overriding the default behavior completely.
//
//	The other language elements (enums, enum values, events, delegates
//	and variables) are currently handled by hard-wired code. New rule
//	types *may* be added to GhostDoc in future versions, but for 2.0
//	the present solution turned out to be good enough for a start.
//
namespace Demo
{
    /// <summary>
    /// </summary>
    /// <typeparam name="U"></typeparam>
    /// <typeparam name="V"></typeparam>
 	public class NewIn200<U,V> // New: (empty) summary for classes, with typeparameter support
	{
        /// <summary>
        /// </summary>
        /// <param name="someParameter"></param>
        /// <typeparam name="T"></typeparam>
 		public void DoSomething<T>( int someParameter ) // type parameters with a single-char name
		{
		}

        /// <summary>
        /// </summary>
        /// <param name="someParameter"></param>
        /// <typeparam name="T"></typeparam>
        public void DoSomething<T>(string someParameter) // type parameters with a single-char name
        {
        }

        /// <summary>
        /// </summary>
        /// <param name="someParameter"></param>
        /// <typeparam name="T"></typeparam>
        public void DoSomething<T>(List<string> someParameter) // type parameters with a single-char name
        {
        }

        /// <summary>
        /// </summary>
        /// <param name="someParameter"></param>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
 		public void DoSomething<TKey, TValue>( int someParameter ) // type parameters with a longer name, starting with a T
		{
		}

        /// <summary>
        /// Occurs when
        /// </summary>
		public event EventHandler<EventArgs> SomethingHappened; // basic support for events

        /// <summary>
        /// </summary>
        /// <param name="e"></param>
		protected virtual void OnSomethingHappened( EventArgs e )
		{
			EventHandler<EventArgs> handler = this.SomethingHappened;
			if (handler != null) handler( this, e );
		}

        /// <summary>
        /// </summary>
        /// <param name="firstValue"></param>
        /// <param name="secondValue"></param>
        /// <returns></returns>
		public static NewIn200<U, V> operator +( NewIn200<U, V> firstValue, NewIn200<U, V> secondValue ) // basic support for operators
		{
			return null;
		}

        /// <summary>
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
		public static explicit operator string( NewIn200<U, V> value ) // support for explicit conversions
		{
			return string.Empty;
		}

        /// <summary>
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
		public static implicit operator int( NewIn200<U, V> value ) // support for implicit conversions
		{
			return 0;
		}
	}

    /// <summary>
    /// </summary>
    /// <typeparam name="TKey"></typeparam>
    /// <typeparam name="TValue"></typeparam>
	public class AnotherClass<TKey, TValue>  // support for type parameters  with a longer name, starting with a T
	{
	}

    /// <summary>
    /// </summary>
    /// <typeparam name="TKey"></typeparam>
    /// <typeparam name="TValue"></typeparam>
	public interface IMyInterface<TKey, TValue>  // ...same for interfaces
	{
	}

    /// <summary>
    /// </summary>
    /// <typeparam name="TKey"></typeparam>
    /// <typeparam name="TValue"></typeparam>
	public struct MyStruct<TKey, TValue>  // ...same for structs
	{
	}

    /// <summary>
    /// </summary>
	public enum MyEnum // empty summary for enums and their values
	{
        /// <summary>
        /// </summary>
		FirstValue,
        /// <summary>
        /// </summary>
		SecondValue,

        /// <summary>
        /// This one already has some documentation we don't want to lose
        /// </summary>
		ThirdValue,

        /// <summary>
        /// </summary>
		FourthValue
	}
}
